/*
 * Copyright(C) 2003  
 *
 *    , 
 *    .
 *
 *        ,
 * ,    ,
 *     ,
 * ,      
 *     
 *      .
 */

/*!
 * \file $RCSfile$
 * \version $Revision: 188550 $
 * \date $Date:: 2019-02-06 17:46:55 +0300#$
 * \author $Author: ivan $
 *
 * \brief       .
 */

#ifndef __CHAIN_UTIL_H
#define __CHAIN_UTIL_H

#include <set>
#include <string>

#include "asn1type.h"
#include "StoreUtil.h"
#include "ASN1Util.h"

class CertChainBuildFlags
{
public:
    //   CRL      
    static const unsigned REVOCATION_CHECK_END_CERT;
    //   CRL   ,  ,
    //    CRL
    static const unsigned REVOCATION_CHECK_CHAIN;
    //   CRL   Root  ,  
    //    CRL. Root    
    // CRL    .
    static const unsigned REVOCATION_CHECK_CHAIN_EXCLUDE_ROOT;
    static const unsigned REVOCATION_CHECK_MASK;
    //        
    //      
    static const unsigned DISABLE_PASS1_QUALITY_FILTERING;
    //   CRL  ,    
    static const unsigned REVOCATION_CHECK_CACHE_ONLY;
    //  timeout   CRL
    static const unsigned REVOCATION_ACCUMULATIVE_TIMEOUT;
    //      
    //    
    static const unsigned CACHE_ONLY_URL_RETRIEVAL;
    unsigned flags;
};

class TrustStatus
{
public:
    // , ,  
    explicit TrustStatus() : value( 0 ) {}
    TrustStatus( const TrustStatus& src ) : value( src.value ) {}
    TrustStatus( unsigned src ) : value( src ) {}

    //   
    unsigned get_unsigned() const { return value; }

    //  CryptoAPI- 
    DWORD toCERT_TRUST_STATUS() const { return value; }
    DWORD toRevocationResult() const;

    // 
    int compare( const TrustStatus &right ) const;

    //   
    std::string toString() const;
    std::wstring toWString() const;
    static TrustStatus fromString( const char *str );

    // 
    static const unsigned CT_NO_ERROR;
    static const unsigned IS_NOT_TIME_VALID;
    static const unsigned IS_REVOKED;
    static const unsigned IS_NOT_SIGNATURE_VALID;
    static const unsigned IS_NOT_VALID_FOR_USAGE;
    static const unsigned IS_UNTRUSTED_ROOT;
    static const unsigned INVALID_BASIC_CONSTRAINTS;
    static const unsigned IS_PARTIAL_CHAIN;
    static const unsigned REVOCATION_STATUS_UNKNOWN;

    //    
    static const unsigned VALUES[];
    static const char *STRINGS[];
#ifdef UNIX
    static const char *WSTRINGS[];
#else /* UNIX */
    static const wchar_t *WSTRINGS[];
#endif /* UNIX */
    static const size_t VALUE_LENGTH;

private:
    int compare( const TrustStatus &src, unsigned flag ) const;
    unsigned value;
};

//   
inline bool operator== ( const TrustStatus &left, const TrustStatus &right )
{ return left.compare( right ) == 0; }

inline bool operator!= ( const TrustStatus &left, const TrustStatus &right )
{ return left.compare( right ) != 0; }

//   ,     
inline bool operator> ( const TrustStatus &left, const TrustStatus &right )
{ return left.compare( right ) > 0; }

//   
class CertChainBuilder;

extern const ASN1OBJID ANY_POLICY_OID;

//     
//  :
// -  : 
// - CRL store: CA, Root
// - Store: CA
// - Root store: Root
// -  . crl , Trust: 
// - Policy: POLICY_ANY
// -  : REVOCATION_CHECK_CHAIN
class CertCC // CertChainContext
{
public:
    // , , 
    CertCC( bool add_root = true, bool extra_crl_stores = true,
	bool extra_cert_stores = true );
    ~CertCC();

    //  
    void set_usage( PCERT_USAGE_MATCH RequestedUsage );
    void set_date( const FILETIME& date );

    void set_flags( unsigned f );
    void set_blacklist_store( HCERTSTORE hStore );
    void set_timeout( unsigned long time_value);

    void add_stores( const wchar_t *store, bool issystem = false );
    void add_stores( const store_handle& handle );
    void add_crl_stores( const wchar_t *store, bool issystem = false );
    void add_crl_stores( const store_handle& handle );

    //    
    void clear_stores();
    void clear_crl_stores();

    //   

    unsigned get_flags( void ) const;
    bool is_blacklisted( PCCERT_CONTEXT ctx ) const;

    //  
    bool verify_certificate(PCCERT_CONTEXT p);

    //   
    TrustStatus get_status() const;

    BOOL toCERT_CHAIN_CONTEXT(PCERT_CHAIN_CONTEXT*);

protected:
    CertChainBuilder *builder;

private:
    //  
    CertCC &operator=(const CertCC &);
};
#endif // __CHAIN_UTIL_H
