#ifndef CLOUD_API_HEADER_H
#define CLOUD_API_HEADER_H

#include "reader/support.h"

#if !defined( _CLOUD_DECL )
#define _CLOUD_DECL
#endif /* !defined( _CLOUD_DECL ) */

#if defined( __cplusplus )
extern "C" {
#endif /* defined( __cplusplus ) */

DWORD LoadCloudLibrary();

typedef void* TDssCertificatesEnumContext;
typedef void* TCloudServerEnumCtx;

typedef struct {
    const char* szUnique;   //   
    const char* szAuthURL;  //   , .. NULL, ,      szUnique
    const char* szRestURL;  //   , .. NULL, ,      szUnique
    const char* szUsername; //   , .. NULL
    const char* szPassword; //   , .. NULL
} TDssServerInfo;


typedef struct {
    BYTE* certificate;              //    
    size_t certificate_length;      //    
    unsigned certificate_id;        //    DSS
    DWORD key_spec;                 //   (AT_KEYEXCHANGE  AT_SIGNATURE)
    DWORD permissions;              //  
    BYTE* extensions;               //    (   )
    size_t extensions_length;       //    
    char* szFriendlyName;           //   
    size_t szFriendlyNameMaxLength; //   
} TDssCertificateInfo;

/*
*        .
*/
_CLOUD_DECL DWORD SetCloudCertificateToStore(
    const TDssServerInfo* serverInfo,           //    
    const TDssCertificateInfo* certificateInfo, //  ,   
    const char* szContName,                     //  ;       ;
		                                //  NULL;  
    BOOL isLM,                                  //        
    const WCHAR *wszProvName,                   // , .. NULL
    DWORD dwProvType                            // , .. 0
);

/*
*      DSS.
* TDssCertificatesEnumContext       DssCertificatesEnumClose.
*/
_CLOUD_DECL DWORD DssCertificatesEnumOpen(
    const TDssServerInfo* serverInfo,           //    
    TDssCertificatesEnumContext *enumContextPtr //     
);

/*
*      
*     .
*/
_CLOUD_DECL DWORD DssCertificateEnumNext(
    TDssCertificatesEnumContext enumContext, //     
    TDssCertificateInfo* containerInfo       //      
);

/*
*   
*/
_CLOUD_DECL void DssCertificatesEnumClose(
    TDssCertificatesEnumContext enumContext
);

/*
*      DSS-xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx.
*/
_CLOUD_DECL DWORD FormContainerName(
    const TDssServerInfo* serverInfo, //  
    unsigned certificateId, //    DSS
    char *szContName, //  ,       ;
    size_t *contNameSize //      
);

/*
*      
*/
_CLOUD_DECL DWORD DssDropAuthCookies(
    const TDssServerInfo* serverInfo //    
);

/*
*    
*/
_CLOUD_DECL DWORD DssSetConnectionTimeout(
    unsigned short timeout // 
);

/*
*    
*/
_CLOUD_DECL DWORD DssGetConnectionTimeout(
    unsigned short *timeout // 
);

/*
*    .
*/
_CLOUD_DECL DWORD CloudServerSettingsEnumOpen(
    TCloudServerEnumCtx* enum_ctx //  
);

/*
*      .
*/
_CLOUD_DECL DWORD CloudServerSettingsEnumNext(
    TCloudServerEnumCtx enum_ctx, //  
    TDssServerInfo* serverInfo //       -    CloudServerSettingsEnumClose
);

/*
*    .     . 
*/
_CLOUD_DECL void CloudServerSettingsEnumClose(
    TCloudServerEnumCtx enum_ctx //  
);

/*
*    .
*   TDssServerInfo: {unique, auth, rest}, {unique, user, pass}, {unique, auth, rest, user, pass}
*/
_CLOUD_DECL DWORD SaveCloudServerSettings(
    const TDssServerInfo* serverInfo, //   
    BOOL isGlobal //       (  )
);

/*
*    
*/
_CLOUD_DECL DWORD DeleteCloudServerSettings(
    const char *szUnique //   
);

/*
*    
*/
_CLOUD_DECL void UpdateCloudServerSettings();

/*
*        
*/
_CLOUD_DECL DWORD UpdateCloudCertificates(
    const char *szUnique //   
);

#if defined( __cplusplus )
}
#endif /* defined( __cplusplus ) */

#endif // CLOUD_API_HEADER_H

